==> roughpy-0.0.2a0.tar.gz: roughpy-0.0.2a0/scalars/include/roughpy/scalars/scalar_traits.h <==
// Copyright (c) 2023 RoughPy Developers. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
//
// 3. Neither the name of the copyright holder nor the names of its contributors
// may be used to endorse or promote products derived from this software without
// specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef ROUGHPY_SCALARS_SCALAR_TRAITS_H_
#define ROUGHPY_SCALARS_SCALAR_TRAITS_H_

#include "scalar.h"
#include "scalars_fwd.h"

namespace rpy {
namespace scalars {

template <typename T> class scalar_type_trait
{
public:
    using value_type = T;
    using rational_type = T;
    using reference = T &;
    using const_reference = const T &;

    RPY_NO_DISCARD
    static const ScalarType *get_type() noexcept { return ScalarType::of<T>(); }

    RPY_NO_DISCARD
    static Scalar make(value_type &&arg)
    {
        return Scalar(get_type(), std::move(arg));
    }
};

template <typename T> class scalar_type_trait<T &>
{
public:
    using value_type = T;
    using rational_type = T;
    using reference = T &;
    using const_reference = const T &;

    RPY_NO_DISCARD
    static const ScalarType *get_type() noexcept
    {
        return scalar_type_trait<T>::get_type();
    }

    RPY_NO_DISCARD
    static Scalar make(reference arg)
    {
        return Scalar(ScalarPointer(get_type(), &arg));
    }
};

template <typename T> class scalar_type_trait<const T &>
{
public:
    using value_type = T;
    using rational_type = T;
    using reference = T &;
    using const_reference = const T &;

    RPY_NO_DISCARD
    static const ScalarType *get_type() noexcept
    {
        return scalar_type_trait<T>::get_type();
    }

    RPY_NO_DISCARD
    static Scalar make(const_reference arg)
    {
        return Scalar(ScalarPointer(get_type(), &arg));
    }
};

}// namespace scalars
}// namespace rpy

#endif// ROUGHPY_SCALARS_SCALAR_TRAITS_H_

==> roughpy-0.0.4.tar.gz: roughpy-0.0.4/scalars/include/roughpy/scalars/scalar_traits.h <==
// Copyright (c) 2023 RoughPy Developers. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
//
// 3. Neither the name of the copyright holder nor the names of its contributors
// may be used to endorse or promote products derived from this software without
// specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

#ifndef ROUGHPY_SCALARS_SCALAR_TRAITS_H_
#define ROUGHPY_SCALARS_SCALAR_TRAITS_H_

#include "scalar.h"
#include "scalars_fwd.h"

namespace rpy {
namespace scalars {

template <typename T>
class scalar_type_trait
{
public:
    using value_type = T;
    using rational_type = T;
    using reference = T&;
    using const_reference = const T&;

    RPY_NO_DISCARD
    static const ScalarType* get_type() noexcept { return ScalarType::of<T>(); }

    RPY_NO_DISCARD
    static Scalar make(value_type&& arg)
    {
        return Scalar(get_type(), std::move(arg));
    }
};

template <typename T>
class scalar_type_trait<T&>
{
public:
    using value_type = T;
    using rational_type = T;
    using reference = T&;
    using const_reference = const T&;

    RPY_NO_DISCARD
    static const ScalarType* get_type() noexcept
    {
        return scalar_type_trait<T>::get_type();
    }

    RPY_NO_DISCARD
    static Scalar make(reference arg)
    {
        return Scalar(ScalarPointer(get_type(), &arg));
    }
};

template <typename T>
class scalar_type_trait<const T&>
{
public:
    using value_type = T;
    using rational_type = T;
    using reference = T&;
    using const_reference = const T&;

    RPY_NO_DISCARD
    static const ScalarType* get_type() noexcept
    {
        return scalar_type_trait<T>::get_type();
    }

    RPY_NO_DISCARD
    static Scalar make(const_reference arg)
    {
        return Scalar(ScalarPointer(get_type(), &arg));
    }
};

}// namespace scalars
}// namespace rpy

#endif// ROUGHPY_SCALARS_SCALAR_TRAITS_H_
